export const BASE_API_URL = import.meta.env.VITE_API_URL ?? window.location.origin;

export function makeApiUrl(pathnameAndQueryparam: string) {
    return new URL(pathnameAndQueryparam, BASE_API_URL).toString();
}

export class ApiError extends Error {
    constructor(public readonly status: number, message: string) {
        super(message);
    }
}

export async function fetcher(url: string, init?: RequestInit) {
    const u = makeApiUrl(url);
    const res = await fetch(u, init);
    if (!res.ok) {
        throw new ApiError(res.status, await res.text());
    }
    return res.json();
}
